@@ -36,7 +36,7 @@ import java.util.*;
 /**
  * Command-line compiler (aka. <tt>groovyc</tt>).
  *
- * @version $Id: FileSystemCompiler.java 21452 2011-02-03 12:02:48Z ait $
+ * @version $Id$
  */
 public class FileSystemCompiler {
     private final CompilationUnit unit;
@@ -204,7 +204,7 @@ public class FileSystemCompiler {
                     errors = true;
                 }
             } else {
-                fileList.addAll(Arrays.asList(filenames));
+                fileList.add(filename);
             }
         }
         if (errors) {
